import { ISignal, Track } from '../ISignal';
import { ISignalEventsObserver } from '../ISignalEventsObserver';
import { HMSConnectionRole } from '../../connection/model';
import HMSMessage from '../../interfaces/message';
export default class JsonRpcSignal implements ISignal {
    private readonly TAG;
    readonly observer: ISignalEventsObserver;
    /**
     * Sometimes before [join] is completed, there could be a lot of trickles
     * Sending [HMSTrickle]` before [join] web socket message leads to
     * error: [500] no rtc transport exists for this Peer
     *
     * We keep a list of pending trickles and send them immediately after [join]
     * is done.
     */
    private isJoinCompleted;
    private pendingTrickle;
    private socket;
    private callbacks;
    constructor(observer: ISignalEventsObserver);
    private call;
    private notify;
    open(uri: string): Promise<void>;
    close(): Promise<void>;
    join(name: string, data: string, offer: RTCSessionDescriptionInit, disableVidAutoSub: boolean): Promise<RTCSessionDescriptionInit>;
    trickle(target: HMSConnectionRole, candidate: RTCIceCandidateInit): void;
    offer(desc: RTCSessionDescriptionInit, tracks: Map<string, any>): Promise<RTCSessionDescriptionInit>;
    answer(desc: RTCSessionDescriptionInit): void;
    trackUpdate(tracks: Map<string, Track>): void;
    broadcast(message: HMSMessage): void;
    recordStart(): void;
    recordEnd(): void;
    leave(): void;
    analytics(): void;
    private onMessageHandler;
}
